INPUT_IS_FILE_PATH_LIST = "__INPUT_IS_FILE_PATH_LIST__"
OUTPUT_IS_EXCEL_FILE = "__OUTPUT_IS_EXCEL_FILE__"
SKIP_CONVERT_TO_JSON = "__SKIP_CONVERT_TO_JSON__"


GRCh37 = "GRCh37"
GRCh38 = "GRCh38"
BUILDS = [GRCh37, GRCh38]
